package ru.amse.fedorov.plainsvg.gui.actions.file;

import java.awt.Dimension;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.SVGFilter;
import ru.amse.fedorov.plainsvg.gui.actions.ComponentAction;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

public abstract class AbstractSaveAction extends ComponentAction {

	private static final long serialVersionUID = 1L;

	private final JFileChooser chooser = new JFileChooser();

	private DocumentBuilder builder;

	public AbstractSaveAction(GraphicsComponent component) {
		super(component);
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			builder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			JOptionPane.showMessageDialog(null, e);
		}
		chooser.setFileFilter(new SVGFilter());
	}

	public AbstractSaveAction(GraphicsComponent component, String name) {
		this(component);
		putValue(Action.NAME, name);
	}

	private void trySaveToFile(File f) throws FileNotFoundException,
			TransformerConfigurationException,
			TransformerFactoryConfigurationError, TransformerException,
			IOException {
		boolean saveFlag = (f != null);
		int replaceResult = JOptionPane.YES_OPTION;
		do {
			if (f == null) {
				if (chooser.showSaveDialog(null) != JFileChooser.APPROVE_OPTION) {
					return;
				}
				f = chooser.getSelectedFile();
				if (!f.getName().endsWith(".svg")) {
					f = new File(f.getAbsolutePath() + ".svg");
				}
				getComponent().setFile(f);
			}
			if (f.exists() && !saveFlag) {

				replaceResult = JOptionPane.showConfirmDialog(null,
						"The file '" + f.getName() + "' already exists.\n"
								+ "Do you want to overwrite it?", "Save file",
						JOptionPane.YES_NO_CANCEL_OPTION);

				if (replaceResult == JOptionPane.CANCEL_OPTION) {
					return;
				}
			}
			if (replaceResult == JOptionPane.NO_OPTION) {
				f = null;
			}
		} while (replaceResult == JOptionPane.NO_OPTION);
		OutputStream stream = null;
		try {
			stream = new FileOutputStream(f);
			Document doc = buildDocument();
			Transformer t = TransformerFactory.newInstance().newTransformer();
			t.setOutputProperty(OutputKeys.INDENT, "yes");
			t.transform(new DOMSource(doc), new StreamResult(stream));
			getComponent().setModelChanged(false);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e);
		} finally {
			if (stream != null) {
				stream.close();
			}
		}
	}

	private Document buildDocument() {
		Document doc = builder.newDocument();
		Element svgElement = doc.createElement("svg");
		doc.appendChild(svgElement);
		double width = getComponent().getRealWidth();
		double height = getComponent().getRealHeight();
		svgElement.setAttribute("width", "" + width + "px");
		svgElement.setAttribute("height", "" + height + "px");
		svgElement.setAttribute("xmlns", "http://www.w3.org/2000/svg");
		svgElement.setAttribute("version", "1.2");
		svgElement.setAttribute("baseProfile", "tiny");

		for (int i = 0; i < getComponent().getElementList().size(); i++) {
			ElementPresentation<?> pres = getComponent().getElementList()
					.get(i);
			svgElement.appendChild(pres.getElement(doc));
		}
		return doc;
	}

	protected void saveToFile(File f)
			throws TransformerFactoryConfigurationError {
		try {
			trySaveToFile(f);
		} catch (TransformerException e1) {
			JOptionPane.showMessageDialog(null,
					"Exceptional condition occured "
							+ "during the transformation process");
		} catch (IOException e2) {
			JOptionPane.showMessageDialog(null, e2);
		}
	}

	protected boolean proposeSave() {
		if (getComponent().isModelChanged()) {
			int result = JOptionPane.showConfirmDialog(null,
					"File has been modified. Save changes?", "Save file",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE);
			switch (result) {
			case JOptionPane.YES_OPTION:
				saveToFile(getComponent().getFile());
				getComponent().setSize(new Dimension(10, 10));
				getComponent().setVisible(false);
				return true;
			case JOptionPane.NO_OPTION:
				return true;
			default:
				return false;
			}
		}
		return true;
	}
}
