package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.NotifyingAffineTransform;
import ru.amse.fedorov.plainsvg.model.elements.SVGEllipse;

/**
 * Wraps 2d-ellipse and svg-ellipse and realises its' connection.
 * 
 * @author Alexey Fedorov
 */
public final class EllipsePresentation extends RectangularShapePresentation<SVGEllipse> {

	/*
	 * Wrapped svg-ellipse.
	 */
	private final SVGEllipse svgEllipse;

	private final Ellipse2D ellipse2D = new Ellipse2D.Double();

	/*
	 * If one of svg-elements contains point, in which mouse had been clicked,
	 * its' minimal circumscribing rectangle with sides encamping parallel with
	 * system axes selects.
	 */
	private final Rectangle2D minRect = null;

	/**
	 * Creates envelope for ellipse using given params.
	 * 
	 * @param ellipse
	 *            wrapped svg-ellipse
	 */
	public EllipsePresentation(SVGEllipse ellipse, GraphicsComponent comp) {
		super(comp);
		this.svgEllipse = ellipse;
		double x = Math.min(svgEllipse.getX1(), svgEllipse.getX2());
		double y = Math.min(svgEllipse.getY1(), svgEllipse.getY2());
		double w = Math.abs(svgEllipse.getX1() - svgEllipse.getX2());
		double h = Math.abs(svgEllipse.getY1() - svgEllipse.getY2());
		ellipse2D.setFrame(x, y, w, h);
	}

	/**
	 * @return inctance of ellipse2D
	 */
	@Override
	public Shape getShape() {
		return ellipse2D;
	}

	/**
	 * @return minimal circumscribed rectangle
	 */
	@Override
	public Rectangle2D getMinRect() {
		NotifyingAffineTransform at = getTransform();

		AffineTransform inverted = new AffineTransform(at);

		try {
			inverted.invert();
		} catch (NoninvertibleTransformException e) {
			assert false;
		}

		double xx = inverted.getScaleX();
		double xy = inverted.getShearX();
		double yx = inverted.getShearY();
		double yy = inverted.getScaleY();

		final double det = xx * yy - xy * yx;

		if (Math.abs(det) <= Double.MIN_VALUE) {
			return minRect;
		}

		final double a = svgEllipse.getRadiusX();
		final double b = svgEllipse.getRadiusY();

		final double s = b * b * xx * xy + a * a * yx * yy;

		final double z1 = Math.hypot(b * xy, a * yy);
		final double z2 = Math.hypot(a * yx, b * xx);

		Point2D[] p = new Point2D[4];
		p[0] = new Point2D.Double(z1 / det, -s / (z1 * det));
		p[1] = new Point2D.Double(-p[0].getX(), -p[0].getY());
		p[2] = new Point2D.Double(-s / (z2 * det), z2 / det);
		p[3] = new Point2D.Double(-p[2].getX(), -p[2].getY());

		Point2D point = at.transform(svgEllipse.getCenter(), null);

		for (int i = 0; i < p.length; i++) {
			p[i].setLocation(p[i].getX() + point.getX(), p[i].getY()
					+ point.getY());
		}

		double minX = p[0].getX();
		double minY = p[0].getY();
		double maxX = p[0].getX();
		double maxY = p[0].getY();

		for (int i = 1; i < p.length; i++) {
			minX = Math.min(minX, p[i].getX());
			minY = Math.min(minY, p[i].getY());
			maxX = Math.max(maxX, p[i].getX());
			maxY = Math.max(maxY, p[i].getY());
		}

		if (minRect == null) {
			return new Rectangle2D.Double(minX, minY, maxX - minX, maxY - minY);
		}
		minRect.setFrame(minX, minY, maxX - minX, maxY - minY);
		return minRect;
	}

	/**
	 * Determines if rectangle in his start configuration (before affine
	 * transforming) contains point with stated coordinates. Calls from
	 * <b>contains</b>
	 * 
	 * @see ru.amse.fedorov.plainsvg.model.elements.SVGElement
	 * 
	 * @param x
	 *            absciss of point
	 * @param y
	 *            ordinate of point
	 * @return <b>true</b> if current element contains point with <b>x</b> and
	 *         <b>y</b> coordinates and <b>false</b> if not
	 */
	@Override
	protected boolean containsUntransformed(Point2D p, double scale) {
		double px = p.getX();
		double py = p.getY();
		double a = (svgEllipse.getCenter().getX() - px)
				/ (svgEllipse.getRadiusX() + svgEllipse.getStrokeWidth() / 2);
		double b = (svgEllipse.getCenter().getY() - py)
				/ (svgEllipse.getRadiusY() + svgEllipse.getStrokeWidth() / 2);
		return (a * a + b * b < 1);
	}

	@Override
	public SVGEllipse getModelElement() {
		return svgEllipse;
	}

	@Override
	protected void createElement(Element element) {
		super.createElement(element);
		element.setAttribute("cx", "" + getModelElement().getCenter().getX());
		element.setAttribute("cy", "" + getModelElement().getCenter().getY());
		element.setAttribute("rx", "" + getModelElement().getRadiusX());
		element.setAttribute("ry", "" + getModelElement().getRadiusY());
	}

	@Override
	protected String getName() {
		return "ellipse";
	}

}
