package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.geom.GeomUtils;
import ru.amse.fedorov.plainsvg.model.elements.SVGLine;
import ru.amse.fedorov.plainsvg.presentation.markers.line.Point1Marker;
import ru.amse.fedorov.plainsvg.presentation.markers.line.Point2Marker;

/**
 * Wraps 2d-line and svg-line and realises its' connection.
 * 
 * @author Alexey Fedorov
 */
public final class LinePresentation extends StrokedPresentation<SVGLine> {

	/*
	 * Wrapped svg-line
	 */
	private final SVGLine line;

	/*
	 * If one of svg-elements contains point, in which mouse had been clicked, //
	 * its' minimal circumscribing rectangle with sides encamping parallel //
	 * with system axes selects.
	 */
	private final Rectangle2D minRect = null;

	/**
	 * Creates envelope for line using given params.
	 * 
	 * @param ellipse
	 *            wrapped svg-line
	 */
	public LinePresentation(SVGLine line) {
		this.line = line;
	}

	@Override
	public void addAllMarkers() {
		getMarkers().add(new Point1Marker(this));
		getMarkers().add(new Point2Marker(this));
	}

	/*
	 * return inctance of line2D
	 */
	@Override
	public Shape getShape() {
		return line.getLine();
	}

	/**
	 * @see ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation
	 */
	@Override
	public Rectangle2D getMinRect() {
		AffineTransform at = getModelElement().getTransform();
		Point2D p1 = at.transform(line.getPoint1(), null);
		Point2D p2 = at.transform(line.getPoint2(), null);

		double minX = Math.min(p1.getX(), p2.getX());
		double minY = Math.min(p1.getY(), p2.getY());
		double maxX = Math.max(p1.getX(), p2.getX());
		double maxY = Math.max(p1.getY(), p2.getY());

		if (minRect == null) {
			return new Rectangle2D.Double(minX, minY, maxX - minX, maxY - minY);
		}
		minRect.setFrame(minX, minY, maxX - minX, maxY - minY);
		return minRect;
	}

	/**
	 * Determines if line in his start configuration (before affine
	 * transforming) contains point with stated coordinates. Calls from
	 * <b>contains</b>.
	 * 
	 * @see ru.amse.fedorov.plainsvg.model.elements.SVGElement
	 * 
	 * @param x
	 *            absciss of point
	 * @param y
	 *            ordinate of point
	 * @return <b>true</b> if current element contains point with <b>x</b> and
	 *         <b>y</b> coordinates and <b>false</b> if not
	 */
	@Override
	protected boolean containsUntransformed(Point2D p, double scale) {
		double minDist = 10 / scale;
		double eps = 0.00000001;
		double x1 = line.getPoint1().getX();
		double y1 = line.getPoint1().getY();
		double x2 = line.getPoint2().getX();
		double y2 = line.getPoint2().getY();
		double s = Math.max(line.getStrokeWidth(), minDist) / 2;
		double l = Point2D.distance(x1, y1, x2, y2);
		double t = 2 * s + l;
		double d1 = (x1 - x2) * s / l;
		double d2 = (y1 - y2) * s / l;

		Point2D pa = new Point2D.Double(x1 + d1, y1 + d2);
		Point2D pb = new Point2D.Double(x2 - d1, y2 - d2);
		Point2D pa1 = new Point2D.Double(pa.getX() + d2, pa.getY() - d1);
		Point2D pa2 = new Point2D.Double(pa.getX() - d2, pa.getY() + d1);
		Point2D pb1 = new Point2D.Double(pb.getX() + d2, pb.getY() - d1);
		Point2D pb2 = new Point2D.Double(pb.getX() - d2, pb.getY() + d1);

		d1 = GeomUtils.dist(p, pa1, pb1) + GeomUtils.dist(p, pa2, pb2);
		d2 = GeomUtils.dist(p, pa1, pa2) + GeomUtils.dist(p, pb1, pb2);
		return (Math.abs(d1 - 2 * s) < eps) && (Math.abs(d2 - t) < eps);
	}

	/**
	 * Repaints image graphics component if need.
	 */
	@Override
	public void paint(Graphics2D graphics, double scale) {
		float st = (float) line.getStrokeWidth();
		graphics.setStroke(new BasicStroke(st));
		graphics.setPaint(line.getStroke());
		graphics.draw(getShape());
	}

	/**
	 * @return svg-shape, wrapped in this element
	 */
	@Override
	public SVGLine getModelElement() {
		return line;
	}

	@Override
	public boolean hasFill() {
		return false;
	}

	@Override
	public void setDiagonal(Point2D p1, Point2D p2) {
		line.setLine(p1, p2);
	}

	@Override
	protected void createElement(Element element) {
		super.createElement(element);
		element.setAttribute("x1", "" + getModelElement().getPoint1().getX());
		element.setAttribute("y1", "" + getModelElement().getPoint1().getY());
		element.setAttribute("x2", "" + getModelElement().getPoint2().getX());
		element.setAttribute("y2", "" + getModelElement().getPoint2().getY());
	}

	@Override
	protected String getName() {
		return "line";
	}

}
