package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGRectangle;

/**
 * Wraps 2d-rectangle and svg-rectangle and realises its' connection
 * 
 * @author Alexey Fedorov
 */
public final class RectanglePresentation extends
		RectangularShapePresentation<SVGRectangle> {

	/*
	 * Wrapped svg-rectangle.
	 */
	private final SVGRectangle svgRectangle;

	private final Rectangle2D rectangle2D = new Rectangle2D.Double();

	/*
	 * If one of svg-elements contains point, in which mouse had been clicked,
	 * its' minimal circumscribing rectangle with sides encamping parallel with
	 * system axes selects.
	 */
	private final Rectangle2D minRect = new Rectangle2D.Double();

	/**
	 * Creates envelope for rectangle using given params.
	 * 
	 * @param ellipse
	 *            wrapped svg-rectangle
	 */
	public RectanglePresentation(SVGRectangle rect, GraphicsComponent comp) {
		super(comp);
		this.svgRectangle = rect;
		double x = svgRectangle.getX();
		double y = svgRectangle.getY();
		double w = svgRectangle.getWidth();
		double h = svgRectangle.getHeight();
		rectangle2D.setFrame(x, y, w, h);
	}

	/**
	 * return inctance of 2D-rectangle
	 */
	@Override
	public Shape getShape() {
		return rectangle2D;
	}

	/**
	 * @see ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation
	 */
	@Override
	public Rectangle2D getMinRect() {
		AffineTransform at = getTransform();
		double x1 = svgRectangle.getX();
		double y1 = svgRectangle.getY();
		double x2 = x1 + svgRectangle.getWidth();
		double y2 = y1 + svgRectangle.getHeight();

		Point2D[] p = new Point2D[4];
		p[0] = new Point2D.Double(x1, y1);
		p[1] = new Point2D.Double(x1, y2);
		p[2] = new Point2D.Double(x2, y1);
		p[3] = new Point2D.Double(x2, y2);

		for (int i = 0; i < p.length; i++) {
			p[i] = at.transform(p[i], null);
		}

		double minX = p[0].getX();
		double maxX = p[0].getX();
		double minY = p[0].getY();
		double maxY = p[0].getY();
		for (int i = 1; i < p.length; i++) {
			minX = Math.min(minX, p[i].getX());
			maxX = Math.max(maxX, p[i].getX());
			minY = Math.min(minY, p[i].getY());
			maxY = Math.max(maxY, p[i].getY());
		}
		minRect.setFrame(minX, minY, maxX - minX, maxY - minY);
		return minRect;

	}

	/**
	 * Determines if rectangle in his start configuration (before affine
	 * transforming) contains point with stated coordinates. Calls from
	 * <b>contains</b>.
	 * 
	 * @see ru.amse.fedorov.plainsvg.model.elements.SVGElement
	 * 
	 * @param x
	 *            absciss of point
	 * @param y
	 *            ordinate of point
	 * @return <b>true</b> if current element contains point with <b>x</b> and
	 *         <b>y</b> coordinates and <b>false</b> if not
	 */
	@Override
	public boolean containsUntransformed(Point2D p, double scale) {
		double px = p.getX();
		double py = p.getY();
		double s = svgRectangle.getStrokeWidth() / 2;
		return (svgRectangle.getX() - s <= px)
				&& (px <= svgRectangle.getX() + svgRectangle.getWidth() + s)
				&& (svgRectangle.getY() - s <= py)
				&& (py <= svgRectangle.getY() + svgRectangle.getHeight() + s);
	}

	@Override
	public SVGRectangle getModelElement() {
		return svgRectangle;
	}

	@Override
	protected void createElement(Element element) {
		super.createElement(element);
		element.setAttribute("x", "" + getModelElement().getX());
		element.setAttribute("y", "" + getModelElement().getY());
		element.setAttribute("width", "" + getModelElement().getWidth());
		element.setAttribute("height", "" + getModelElement().getHeight());
	}

	@Override
	protected String getName() {
		return "rect";
	}

}
